@@ -4,7 +4,7 @@
 #'   rows.
 #' @usage lspec(X = NULL, flim = c(0,22), sxrow = 5, rows = 10, collev = seq(-40, 0, 1), 
 #' ovlp = 50, parallel = 1, wl = 512, gr = FALSE, pal = reverse.gray.colors.2, 
-#' cex = 1, it = "jpeg", flist = NULL, redo = TRUE, path = NULL) 
+#' cex = 1, it = "jpeg", flist = NULL, redo = TRUE, path = NULL, pb = TRUE) 
 #' @param X Data frame with results from \code{\link{manualoc}} or any data frame with columns
 #' for sound file name (sound.files), selection number (selec), and start and end time of signal
 #' (start and end). If given, two red dotted lines are plotted at the 
@@ -41,6 +41,8 @@
 #'   file in the working directory will be analyzed. Default is \code{FALSE}.
 #' @param path Character string containing the directory path where the sound files are located. 
 #' If \code{NULL} (default) then the current working directory is used.
+#' @param pb Logical argument to control progress bar. Default is \code{TRUE}. Note that progress bar is only used
+#' when parallel = 1.
 #' @return image files with spectrograms of whole sound files in the working directory. Multiple pages
 #' can be returned, depending on the length of each sound file. 
 #' @export
@@ -49,8 +51,8 @@
 #'   the name of the sound files and the "page" number (p1-p2...) at the upper 
 #'   right corner of the image files. If results from \code{\link{manualoc}} are 
 #'   supplied (or an equivalent data frame), the function delimits and labels the selections. 
-#'   This function aims to facilitate visual classification of vocalization units and the 
-#'   analysis of animal vocal sequences.
+#'   This function aims to facilitate visual inspection of multiple files as well as visual classification 
+#'   of vocalization units and the analysis of animal vocal sequences.
 #' @examples
 #' \dontrun{
 #' # First create empty folder
@@ -69,15 +71,16 @@
 #' getwd()
 #' }
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu})
-#last modification on jul-5-2016 (MAS)
+#last modification on nov-12-2016 (MAS)
 
 lspec <- function(X = NULL, flim = c(0, 22), sxrow = 5, rows = 10, collev = seq(-40, 0, 1),  ovlp = 50, parallel = 1, 
-                  wl = 512, gr = FALSE, pal = reverse.gray.colors.2, cex = 1, it = "jpeg", flist = NULL, redo = TRUE, path = NULL) {
+                  wl = 512, gr = FALSE, pal = reverse.gray.colors.2, cex = 1, it = "jpeg", flist = NULL, redo = TRUE, path = NULL, pb = TRUE) {
   
   #check path to working directory
-  if(!is.null(path)) 
-    if(class(try(setwd(path), silent = T)) == "try-error") stop("'path' provided does not exist") else 
-      setwd(path) #set working directory
+  if(!is.null(path))
+  {wd <- getwd()
+  if(class(try(setwd(path), silent = TRUE)) == "try-error") stop("'path' provided does not exist") else 
+    setwd(path)} #set working directory
   
   #if sel.comment column not found create it
   if(is.null(X$sel.comment) & !is.null(X)) X<-data.frame(X,sel.comment="")
@@ -150,16 +153,13 @@ lspec <- function(X = NULL, flim = c(0, 22), sxrow = 5, rows = 10, collev = seq(
   #if it argument is not "jpeg" or "tiff" 
   if(!any(it == "jpeg", it == "tiff")) stop(paste("Image type", it, "not allowed"))  
   
+  #wrap img creating function
+  if(it == "jpeg") imgfun <- jpeg else imgfun <- tiff
+  
   #if parallel is not numeric
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
-  #if parallel
-  # if(all(parallel > 1, !Sys.info()[1] %in% c("Linux","Windows"))) {
-  #   parallel <- 1
-  #   message("creating images is not compatible with parallel computing (parallel > 1) in OSX (mac)")
-  # }
-  
   # redo
   if(!redo) 
     files <- files[!gsub(".wav$","", list.files(pattern = ".wav$", ignore.case = TRUE),ignore.case = TRUE) %in% 
@@ -185,12 +185,13 @@ lspec <- function(X = NULL, flim = c(0, 22), sxrow = 5, rows = 10, collev = seq(
     dur <- length(rec@left)/rec@samp.rate #set duration    
     
     if(!is.null(malo)) ml <- ml[ml$sound.files == z,] #subset X data
+    
     #loop over pages 
-    for (j in 1:ceiling(dur/(li*sl))){
-      if(it == "tiff") tiff(filename = paste(substring(z, first = 1, last = nchar(z)-4), "-p", j, ".tiff", sep = ""),  
-           res = 160, units = "in", width = 8.5, height = 11) else
-             jpeg(filename = paste(substring(z, first = 1, last = nchar(z)-4), "-p", j, ".jpeg", sep = ""),  
-           res = 160, units = "in", width = 8.5, height = 11)
+    no.out <-lapply(1:ceiling(dur/(li*sl)), function(j)  
+      {
+       imgfun(filename = paste0(substring(z, first = 1, last = nchar(z)-4), "-p", j, ".", it),  
+           res = 160, units = "in", width = 8.5, height = 11) 
+      
       par(mfrow = c(li,  1), cex = 0.6, mar = c(0,  0,  0,  0), oma = c(2, 2, 0.5, 0.5), tcl = -0.25)
       
       #creates spectrogram rows
@@ -243,40 +244,54 @@ lspec <- function(X = NULL, flim = c(0, 22), sxrow = 5, rows = 10, collev = seq(
       }
       dev.off() #reset graphic device
     }
+    )
     }
     
    #Apply over each sound file
-    # Run parallel in windows
-    if(parallel > 1) {if(Sys.info()[1] == "Windows") {
-      
-      z <- NULL
-      
-      cl <- parallel::makeCluster(parallel)
-      
-      doParallel::registerDoParallel(cl)
-      
-      
-      sp <- foreach::foreach(z = files) %dopar% {
-          lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X)
-      }
-      
-#       sp <- parallel::parLapply(cl, files, function(z)
-#       {
-#         lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X)
-#       })
-      
-      parallel::stopCluster(cl)
-      
-    } else {    # Run parallel in other operating systems
-      
-      sp <- parallel::mclapply(files, function(z) {
-        lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X)
-      })
-    }
-    }
-    else {sp <- pbapply::pblapply(files, function(z) 
-      lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X))
-    }
-     
+   # Run parallel in windows
+   if(parallel > 1) {
+     if(Sys.info()[1] == "Windows") {
+       
+       z <- NULL
+       
+       cl <- parallel::makeCluster(parallel)
+       
+       doParallel::registerDoParallel(cl)
+       
+       sp <- foreach::foreach(z = files) %dopar% {
+         lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X)
+       }
+       
+       parallel::stopCluster(cl)
+       
+     } 
+     if(Sys.info()[1] == "Linux") {    # Run parallel in Linux
+       
+       sp <- parallel::mclapply(files, function (z) {
+         lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X)
+       })
+     }
+     if(!any(Sys.info()[1] == c("Linux", "Windows"))) # parallel in OSX
+     {
+       cl <- parallel::makeForkCluster(getOption("cl.cores", parallel))
+       
+       doParallel::registerDoParallel(cl)
+       
+       sp <- foreach::foreach(z = files) %dopar% {
+         lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X)
+       }
+       
+       parallel::stopCluster(cl)
+       
+     }
+   }
+   else {
+     if(pb)
+     sp <- pbapply::pblapply(files, function(z) 
+       lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X)) else
+         sp <- lapply(files, function(z) 
+           lspecFUN(z = z, fl = flim, sl = sxrow, li = rows, ml = manloc, malo = X))
+   }
+   if(!is.null(path)) on.exit(setwd(wd))       
 }
 
