@@ -1,11 +1,12 @@
 #' Measure acoustic parameters in batches of sound files
 #'
-#' \code{specan} measures 22 acoustic parameters on acoustic signals for which the start and end times 
+#' \code{specan} measures acoustic parameters on acoustic signals for which the start and end times 
 #' are provided. 
-#' @usage specan(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast = TRUE, path = NULL)
+#' @usage specan(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast = TRUE, path = NULL, 
+#' pb = TRUE)
 #' @param X Data frame with the following columns: 1) "sound.files": name of the .wav 
 #' files, 2) "sel": number of the selections, 3) "start": start time of selections, 4) "end": 
-#' end time of selections. The ouptut of \code{\link{manualoc}} or \code{\link{autodetec}} can 
+#' end time of selections. The ouptut of \code{\link{manualoc}} or \code{\link{autodetec}} can
 #' be used as the input data frame.
 #' @param bp Numeric vector of length 2 giving the lower and upper limits of the 
 #' frequency bandpass filter (in kHz). Default is c(0, 22).
@@ -19,6 +20,8 @@
 #' substantially increases performance (~9 times faster).
 #' @param path Character string containing the directory path where the sound files are located. 
 #' If \code{NULL} (default) then the current working directory is used.
+#' @param pb Logical argument to control progress bar. Default is \code{TRUE}. Note that progress bar is only used
+#' when parallel = 1.
 #' @return Data frame with the following acoustic parameters: 
 #' \itemize{
 #'    \item \code{duration}: length of signal
@@ -43,13 +46,16 @@
 #'    \item \code{maxdom}: maximum of dominant frequency measured across acoustic signal 
 #'    \item \code{dfrange}: range of dominant frequency measured across acoustic signal 
 #'    \item \code{modindx}: modulation index. Calculated as the accumulated absolute 
-#'      difference between adjacent measurements of fundamental frequencies divided
-#'      by the frequency range
+#'      difference between adjacent measurements of dominant frequencies divided
+#'      by the dominant frequency range
+#'    \item \code{startdom}:  dominant frequency measurement at the start of the signal 
+#'    \item \code{enddom}: dominant frequency measurement at the end of the signal 
+#'    \item \code{dfslope}: slope of the change in dominant through time ([enddom-startdom]/duration)  
 #' }
 #' @export
 #' @name specan
 #' @details The ouptut of \code{\link{manualoc}} or \code{\link{autodetec}} can be used 
-#'  directly without any additional modification. The function measures 22 acoustic parameters on 
+#'  directly without any additional modification. The function measures 25 acoustic parameters (if fast = \code{TRUE}) on 
 #'  each selection in the data frame. Most parameters are produced internally by 
 #'  \code{\link[seewave]{specprop}}, \code{\link[seewave]{fpeaks}}, \code{\link[seewave]{fund}},
 #'  and \code{\link[seewave]{dfreq}} from the package seewave. 
@@ -75,11 +81,14 @@
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu}) and Grace Smith Vidaurre
 #last modification on jul-5-2016 (MAS)
 
-specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast = TRUE, path = NULL){
+specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast = TRUE, 
+                   path = NULL, pb = TRUE){
   
   #check path to working directory
   if(!is.null(path))
-  {if(class(try(setwd(path), silent = T)) == "try-error") stop("'path' provided does not exist") else setwd(path)} #set working directory
+  {wd <- getwd()
+  if(class(try(setwd(path), silent = TRUE)) == "try-error") stop("'path' provided does not exist") else 
+    setwd(path)} #set working directory
   
   #if X is not a data frame
   if(!class(X) == "data.frame") stop("X is not a data frame")
@@ -168,6 +177,10 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
     maxdom <- max(y, na.rm = TRUE)
     dfrange <- (maxdom - mindom)
     duration <- (X$end[i] - X$start[i])
+    startdom <- y[!is.na(y)][1]
+    enddom <- y[!is.na(y)][length(y[!is.na(y)])]
+    dfslope <- (enddom -startdom)/duration
+    
     
     #modulation index calculation
     changes <- vector()
@@ -179,9 +192,9 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
     
     #save results
     if(fast) return(data.frame(sound.files = X$sound.files[i], selec = X$selec[i], duration, meanfreq, sd, median, Q25, Q75, IQR, skew, kurt, sp.ent, sfm, mode, 
-                               centroid, meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx)) else
+                               centroid, meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx, startdom, enddom, dfslope)) else
                                  return(data.frame(sound.files = X$sound.files[i], selec = X$selec[i], duration, meanfreq, sd, median, Q25, Q75, IQR, skew, kurt, sp.ent, sfm, mode, 
-                                                   centroid, peakf, meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx))
+                                                   centroid, peakf, meanfun, minfun, maxfun, meandom, mindom, maxdom, dfrange, modindx, startdom, enddom, dfslope))
   }
   
   # Run parallel in windows
@@ -203,13 +216,16 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
     
   } else {    # Run parallel in other operating systems
     
-    sp <- parallel::mclapply(1:nrow(X), function (i) {
+    sp <- parallel::mclapply(1:nrow(X), function(i) {
       spFUN(X = X, i = i, bp = bp, wl = wl, threshold = threshold)
     })
     
   }
   }
-  else {sp <- pbapply::pblapply(1:nrow(X), function(i) spFUN(X, i = i, bp = bp, wl = wl, threshold = threshold))             
+  else {
+    if(pb) 
+      sp <- pbapply::pblapply(1:nrow(X), function(i) spFUN(X, i = i, bp = bp, wl = wl, threshold = threshold)) else
+        sp <- lapply(1:nrow(X), function(i) spFUN(X, i = i, bp = bp, wl = wl, threshold = threshold))        
 
   }
   sp <- do.call(rbind, sp)
@@ -217,4 +233,5 @@ specan <- function(X, bp = c(0,22), wl = 512, threshold = 15, parallel = 1, fast
   row.names(sp) <- 1:nrow(sp)
   
   return(sp)
-}
+  if(!is.null(path)) on.exit(setwd(wd))
+  }
